/*
 * Copyright (c) 2001-2011, Perforce Software, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

extern "C"
{
    #include "php.h"
}

#include "php_perforce.h"

#include "php_p4_output_handler.h"

zend_class_entry *p4_output_handler_interface_ce;
zend_class_entry *p4_output_handler_abstract_ce;

/* P4_OutputHandlerInterface Methods */
static zend_function_entry perforce_p4_output_handler_interface_functions[] = {
    PHP_ABSTRACT_ME(P4_OutputHandlerInterface, outputStat,    NULL)
    PHP_ABSTRACT_ME(P4_OutputHandlerInterface, outputInfo,    NULL)
    PHP_ABSTRACT_ME(P4_OutputHandlerInterface, outputText,    NULL)
    PHP_ABSTRACT_ME(P4_OutputHandlerInterface, outputBinary,  NULL)
    PHP_ABSTRACT_ME(P4_OutputHandlerInterface, outputWarning, NULL)
    PHP_ABSTRACT_ME(P4_OutputHandlerInterface, outputError,   NULL)
    { NULL, NULL, NULL }
};

/* P4_OutputHandlerAbstract Methods */
static zend_function_entry perforce_p4_output_handler_abstract_functions[] = {
    PHP_ME(    P4_OutputHandlerAbstract, outputStat,                NULL, ZEND_ACC_PUBLIC)
    PHP_MALIAS(P4_OutputHandlerAbstract, outputInfo,    outputStat, NULL, ZEND_ACC_PUBLIC)
    PHP_MALIAS(P4_OutputHandlerAbstract, outputText,    outputStat, NULL, ZEND_ACC_PUBLIC)
    PHP_MALIAS(P4_OutputHandlerAbstract, outputBinary,  outputStat, NULL, ZEND_ACC_PUBLIC)
    PHP_MALIAS(P4_OutputHandlerAbstract, outputWarning, outputStat, NULL, ZEND_ACC_PUBLIC)
    PHP_MALIAS(P4_OutputHandlerAbstract, outputError,   outputStat, NULL, ZEND_ACC_PUBLIC)
    { NULL, NULL, NULL }
};

/* Get the zend_class_entry for P4_OutputHandlerInterface. */
zend_class_entry *get_p4_output_handler_interface_ce(void)
{
    return p4_output_handler_interface_ce;
}

/* Get the zend_class_entry for P4_OutputHandlerAbstract. */
zend_class_entry *get_p4_output_handler_abstract_ce(void)
{
    return p4_output_handler_abstract_ce;
}

/* Register the P4_OutputHandler Interface and Abstract. */
void register_p4_output_handler(INIT_FUNC_ARGS)
{
    zend_class_entry ceI, ceA;
    
    REGISTER_PERFORCE_CLASS(ceI, "P4_OutputHandlerInterface", NULL,
        perforce_p4_output_handler_interface_functions, p4_output_handler_interface_ce);
    p4_output_handler_interface_ce->ce_flags |= ZEND_ACC_INTERFACE;
    
    register_p4_output_handler_constants(p4_output_handler_interface_ce);
    
    REGISTER_PERFORCE_CLASS(ceA, "P4_OutputHandlerAbstract", p4_output_handler_interface_ce,
        perforce_p4_output_handler_abstract_functions, p4_output_handler_abstract_ce);
    p4_output_handler_abstract_ce->ce_flags |= ZEND_ACC_ABSTRACT;
}

void register_p4_output_handler_constants(zend_class_entry* ce)
{
    zval *report, *handled, *cancel;
    
    report = (zval*)pemalloc(sizeof(zval), 1);
    INIT_PZVAL(report);
    ZVAL_LONG(report, HANDLER_REPORT);

    handled = (zval*)pemalloc(sizeof(zval), 1);
    INIT_PZVAL(handled);
    ZVAL_LONG(handled, HANDLER_HANDLED);

    cancel = (zval*)pemalloc(sizeof(zval), 1);
    INIT_PZVAL(cancel);
    ZVAL_LONG(cancel, HANDLER_CANCEL);
    
    zend_hash_add(&ce->constants_table, "HANDLER_REPORT", sizeof("HANDLER_REPORT"), 
                  (void*)&report, sizeof(zval*), NULL);
    zend_hash_add(&ce->constants_table, "HANDLER_HANDLED", sizeof("HANDLER_HANDLED"), 
                  (void*)&handled, sizeof(zval*), NULL);
    zend_hash_add(&ce->constants_table, "HANDLER_CANCEL", sizeof("HANDLER_CANCEL"), 
                  (void*)&cancel, sizeof(zval*), NULL);
}

/* {{{ proto string P4_OutputHandlerAbstract::outputStat()
    Simply return HANDLER_REPORT; used as an alias for all output methods. */
PHP_METHOD(P4_OutputHandlerAbstract, outputStat)
{
    RETURN_LONG(HANDLER_REPORT);
}
/* }}} */
